// Copyright (C) 2020-2021  Nexedi SA and Contributors.
//                          Kirill Smelkov <kirr@nexedi.com>
//
// This program is free software: you can Use, Study, Modify and Redistribute
// it under the terms of the GNU General Public License version 3, or (at your
// option) any later version, as published by the Free Software Foundation.
//
// You can also Link and Combine this program with other software covered by
// the terms of any of the Free Software licenses or any of the Open Source
// Initiative approved licenses and Convey the resulting work. Corresponding
// source of such a combination shall include the source code for all other
// software used.
//
// This program is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// See COPYING file for full licensing terms.
// See https://www.nexedi.com/licensing for rationale and options.

package xbtree_test
// ZBlk-related part of δbtail_test

import (
	"context"
	"fmt"

	"lab.nexedi.com/kirr/go123/xerr"
	"lab.nexedi.com/kirr/neo/go/zodb"

	"lab.nexedi.com/nexedi/wendelin.core/wcfs/internal/xbtree"
	"lab.nexedi.com/nexedi/wendelin.core/wcfs/internal/xzodb"
	"lab.nexedi.com/nexedi/wendelin.core/wcfs/internal/zdata"
)

type Tree = xbtree.Tree
type Node = xbtree.Node
type Key  = xbtree.Key

type ZBlk = zdata.ZBlk

// ztreeGetBlk returns ztree[k] and tree path that lead to this block.
// XXX +return blkRevMax and use it ?
func ztreeGetBlk(ctx context.Context, ztree *Tree, k Key) (zblk ZBlk, ok bool, path []Node, err error) {
	path = []Node{}
	xzblk, ok, err := ztree.VGet(ctx, k, func(node Node) {
		path = append(path, node)
	})
	if err != nil {
		return nil, false, nil, err
	}

	if ok {
		zblk, ok = xzblk.(ZBlk)
		if !ok {
			return nil, false, nil, fmt.Errorf("expect ZBlk*; got %s", xzodb.TypeOf(xzblk))  // XXX errctx
		}
	}

	return zblk, ok, path, nil
}


func init() {
	xbtree.ZTreeGetBlkData = ZTreeGetBlkData
	xbtree.ZGetBlkData     = ZGetBlkData
}

// ZTreeGetBlkData returns block data from block pointed to by ztree[k].
func ZTreeGetBlkData(ctx context.Context, ztree *Tree, k Key) (data string, ok bool, path []Node, err error) {
	defer xerr.Contextf(&err, "@%s: tree<%s>: get blkdata from [%d]", ztree.PJar().At(), ztree.POid(), k)

	zblk, ok, path, err := ztreeGetBlk(ctx, ztree, k)
	if err != nil || !ok {
		return "", ok, path, err
	}

	bdata, _, err := zblk.LoadBlkData(ctx)
	if err != nil {
		return "", false, nil, err
	}

	return string(bdata), true, path, nil
}

// ZGetBlkData loads block data from ZBlk object specified by its oid.
func ZGetBlkData(ctx context.Context, zconn *zodb.Connection, zblkOid zodb.Oid) (data string, err error) {
	defer xerr.Contextf(&err, "@%s: get blkdata from obj %s", zconn.At(), zblkOid)

	xblk, err := zconn.Get(ctx, zblkOid)
	if err != nil {
		return "", err
	}
	zblk, ok := xblk.(ZBlk)
	if !ok {
		return "", fmt.Errorf("expect ZBlk*; got %s", xzodb.TypeOf(xblk))
	}
	bdata, _, err := zblk.LoadBlkData(ctx)
	if err != nil {
		return "", err
	}
	return string(bdata), nil
}
